from typing import TypedDict


class NippleOptionDict(TypedDict):
    boyish: bool
    dir: str
    dry: bool
    first_disable: bool
    first_enable: bool
    left: bool
    loli: bool
    low: bool
    repeat: int
    right: bool
    serif_disable: bool
    serif_enable: bool
    wet: bool
